﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Controller : MonoBehaviour {

	public static Controller _;

	List<LevelData> Levels = LevelData.Levels ();
	int m_currentLevel = 0;

	void Awake () {
		//There Can Only Be One
		if (_ == null)
			_ = this;
		else
			Destroy (this.gameObject);
	}
	public GameObject sucessSound;
	public GameObject newPuzzleSound;
	public GameObject xplodeParticles;
	public FadePanel fadePanel;
	public WaveCombine combineWave;
	public WaveDisplay puzzleWave;

	private Vector2 m_hexTarget = Vector2.zero, m_circTarget = Vector2.zero, m_triTarget = Vector2.zero;
	public Vector2 hexTarget { get { return m_hexTarget; } }
	public Vector2 circTarget { get { return m_circTarget; } }
	public Vector2 triTarget { get { return m_triTarget; } }

	public bool hexSolved { get { return (Mathf.Abs ((m_hex - m_hexTarget).magnitude) < 0.05f); } }
	public bool circSolved { get { return (Mathf.Abs ((m_circ - m_circTarget).magnitude) < 0.05f); } }
	public bool triSolved { get { return (Mathf.Abs ((m_tri - m_triTarget).magnitude) < 0.05f); } }

	bool m_newPuzzleCoroutineRunning = false;

	public void GetNewPuzzle(float wait = 0){ if (!m_newPuzzleCoroutineRunning) { StartCoroutine (NewPuzzleTimer (wait)); } }

	public IEnumerator NewPuzzleTimer(float wait = 0){
		m_newPuzzleCoroutineRunning = true;
		if (m_currentLevel > 0) {
			while (wait > .01f) {
				wait -= Time.deltaTime;
				yield return null;
			}
			GameObject ss = GameObject.Instantiate (sucessSound) as GameObject;
			combineWave.gameObject.SetActive (true);
			yield return StartCoroutine (combineWave.Animation ());
			combineWave.gameObject.SetActive (false);
			Destroy (ss);
			GameObject nps = GameObject.Instantiate (newPuzzleSound) as GameObject;
			Destroy (nps, 4f);
			GameObject xplode = GameObject.Instantiate (xplodeParticles) as GameObject;
			Destroy (xplode, 3f);
		}
		hexJem.InitJems ();
		circJem.InitJems ();
		triJem.InitJems ();
		if (m_currentLevel < Levels.Count) {
			m_hexTarget = Levels [m_currentLevel].hexTarget;
			m_circTarget = Levels [m_currentLevel].circTarget;
			m_triTarget = Levels [m_currentLevel].triTarget;

			m_currentLevel++;
		} else if (m_currentLevel < (Levels.Count + 1)) {
			GetGoodRandomPuzzleTargets ();
			//Bad Random Puzzle Targets:
			//m_hexTarget = new Vector2 (Random.Range (-1f, 1f), Random.Range (-1f, 1f));
			//m_circTarget = new Vector2 (Random.Range (-1f, 1f), Random.Range (-1f, 1f));
			//m_triTarget = new Vector2 (Random.Range (-1f, 1f), Random.Range (-1f, 1f));
			m_currentLevel++;
		} else {
			fadePanel.LoadEnd ();
		}

		puzzleWave.DrawWave (WaveGenerator.yValPuzzle);
		m_newPuzzleCoroutineRunning = false;
	}
	float randomSign {get{return Mathf.Sign (Random.value - 0.5f);}}
	void GetGoodRandomPuzzleTargets(){
		int i = Random.Range (0, 6);
		float lowFrequencyAmplitude = Random.Range (0.8f, 1f) * randomSign;
		float lowFrequencyFrequency = Random.Range (-1f, -0.3f);
		float highFrequencyAmplitude = Random.Range (0.5f, 0.75f) * randomSign;
		float highFrequencyFrequency = Random.Range (0.15f, 0.75f);
		switch (i) {
		case 0:
			m_hexTarget = Vector2.zero;
			m_circTarget = new Vector2 (lowFrequencyFrequency, lowFrequencyAmplitude);
			m_triTarget = new Vector2 (highFrequencyFrequency, highFrequencyFrequency);
			break;
		case 1:
			m_hexTarget = new Vector2 (lowFrequencyFrequency, lowFrequencyAmplitude);
			m_circTarget = Vector2.zero;
			m_triTarget = new Vector2 (highFrequencyFrequency, highFrequencyFrequency);
			break;
		case 2:
			m_hexTarget = new Vector2 (lowFrequencyFrequency, lowFrequencyAmplitude);
			m_circTarget = new Vector2 (highFrequencyFrequency, highFrequencyFrequency);
			m_triTarget = Vector2.zero;
			break;
		case 3:
			m_hexTarget = Vector2.zero;
			m_circTarget = new Vector2 (highFrequencyFrequency, highFrequencyFrequency);
			m_triTarget = new Vector2 (lowFrequencyFrequency, lowFrequencyAmplitude);
			break;
		case 4:
			m_hexTarget = new Vector2 (highFrequencyFrequency, highFrequencyFrequency);
			m_circTarget = Vector2.zero;
			m_triTarget = new Vector2 (lowFrequencyFrequency, lowFrequencyAmplitude);
			break;
		case 5:
			m_hexTarget = new Vector2 (highFrequencyFrequency, highFrequencyFrequency);
			m_circTarget = new Vector2 (lowFrequencyFrequency, lowFrequencyAmplitude);
			m_triTarget = Vector2.zero;
			break;
		}
	}

	[Range(0,50)]public float jemRange = 2.5f;
		
	//private float m_amplitude, m_frequency, m_phase;
	//public float amplitude { get { return m_amplitude; } set { m_amplitude = Mathf.Clamp (value, -1f, 1f); UpdateMasterWave (); UpdateTriWave ();} }
	//public float frequency { get { return m_frequency; } set { m_frequency = Mathf.Clamp (value, -1f, 1f); UpdateMasterWave (); UpdateHexWave ();} }
	//public float phase { get { return m_phase; } set { m_phase = Mathf.Clamp (value, -1f, 1f); UpdateMasterWave (); UpdateCircleWave ();} }

	private Vector2 m_hex = Vector2.zero, m_circ = Vector2.zero, m_tri = Vector2.zero;
	public Vector2 hex {
		get { return m_hex; }
		set {
			m_hex = new Vector2 (Mathf.Clamp (value.x, -1f, 1f), Mathf.Clamp (value.y, -1f, 1f));
			UpdateMasterWave ();
			UpdateHexWave ();
		}
	}
	public Vector2 circ {
		get { return m_circ; }
		set {
			m_circ = new Vector2 (Mathf.Clamp (value.x, -1f, 1f), Mathf.Clamp (value.y, -1f, 1f));
			UpdateMasterWave ();
			UpdateCircleWave ();
		}
	}
	public Vector2 tri {
		get { return m_tri; }
		set {
			m_tri = new Vector2 (Mathf.Clamp (value.x, -1f, 1f), Mathf.Clamp (value.y, -1f, 1f));
			UpdateMasterWave ();
			UpdateTriWave ();
		}
	}

	public WaveDisplay masterWave, hexWave, circleWave, triWave;
	public JemSlider hexJem, circJem, triJem;

	public void UpdateMasterWave(){ masterWave.DrawWave (WaveGenerator.yValMaster);    }
	public void UpdateHexWave()   { hexWave.DrawWave    (WaveGenerator.yValHex); }
	public void UpdateCircleWave() { circleWave.DrawWave  (WaveGenerator.yValCirc);     }
	public void UpdateTriWave()  { triWave.DrawWave   (WaveGenerator.yValTri); }

	public void UpdateAllWaves(){
		UpdateMasterWave ();
		UpdateHexWave ();
		UpdateCircleWave ();
		UpdateTriWave ();
	}

	void Start(){
		UpdateAllWaves ();
		GetNewPuzzle ();
	}

	bool Solved(){
		if (hexSolved && circSolved && triSolved)
			return true;
		else
			return false;
	}

	void Update(){
		if (Solved ())
			GetNewPuzzle (3f);
	}

	void Reset(){
	}
}
