﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class JemSlider : Selectable {

	public enum SliderType
	{
		NONE,
		HEXAGON,
		CIRCLE,
		TRIANGLE
	}

	public RectTransform jem;
	public GameObject glowObject;
	public SliderType type = SliderType.NONE;
	public AudioSource sound;

	private bool followCursor = false;
	private Vector2 offset = Vector2.zero;

	public new void Start(){
		InitJems ();
	}
		
	public void InitJems(){
		Vector2 initPosition = Vector2.zero;
		jem.anchoredPosition = initPosition;
		glowObject.SetActive(false);
		SetJemBounds (type);
	}

	public override void OnPointerDown (UnityEngine.EventSystems.PointerEventData eventData)
	{
		base.OnPointerDown (eventData);
		offset = new Vector2 (jem.position.x - Input.mousePosition.x, jem.position.y - Input.mousePosition.y);
		followCursor = true;
	}
	public override void OnPointerUp (UnityEngine.EventSystems.PointerEventData eventData)
	{
		base.OnPointerUp (eventData);
		followCursor = false;
		sound.volume = 0;
	}
		
	public void SetJemBounds(SliderType type){
		//Let's create the boundaries of the Slider!
		Vector2 range = jem.anchoredPosition / (Controller._.jemRange * 100);

		switch (type) {

		case SliderType.HEXAGON:
			Controller._.hex = range;
			break;
		case SliderType.CIRCLE:
			Controller._.circ = range;
			break;
		case SliderType.TRIANGLE:
			Controller._.tri = range;
			break;
		}
	}

	public void TestJemBounds() {
		Vector2 parentSize = jem.parent.GetComponent<RectTransform> ().sizeDelta;
		Vector2 jemPosition = jem.localPosition;

		if (jemPosition.x < (-parentSize.x * 0.5f)) {
			jemPosition.x = -parentSize.x * 0.5f;
		} else if (jemPosition.x > (parentSize.x * 0.5f)) {
			jemPosition.x = parentSize.x * 0.5f;
		}

		if (jemPosition.y < (-parentSize.y * 0.5f)) {
			jemPosition.y = -parentSize.y * 0.5f;
		} else if (jemPosition.y > (parentSize.y * 0.5f)) {
			jemPosition.y = parentSize.y * 0.5f;
		}

		jem.localPosition = jemPosition;
	}

	public void CheckGlow(SliderType type){
		bool glow = false;
		switch (type) {

		case SliderType.HEXAGON:
			glow = Controller._.hexSolved;
			break;
		case SliderType.CIRCLE:
			glow = Controller._.circSolved;
			break;
		case SliderType.TRIANGLE:
			glow = Controller._.triSolved;
			break;
		}

		glowObject.SetActive (glow);
	}

	public void MoveJem(SliderType type){
		jem.position = new Vector2 (Input.mousePosition.x, Input.mousePosition.y) + offset;
	}
		
	//Runs every frame
	void Update () {
		
		if (followCursor) {
			SetJemBounds(type);
			MoveJem (type);
			TestJemBounds ();
			SetSoundParams ();
		}
		TestJemBounds ();



		CheckGlow (type);
	}

	void SetSoundParams(){
		float vol = 0;
		switch (type) {
		case SliderType.HEXAGON:
			vol = (1 - (Controller._.hex - Controller._.hexTarget).magnitude) * 0.25f;
			break;
		case SliderType.CIRCLE:
			vol = (1 - (Controller._.circ - Controller._.circTarget).magnitude);
			break;
		case SliderType.TRIANGLE:
			vol = (1 - (Controller._.tri - Controller._.triTarget).magnitude) * 0.25f;
			break;
		}
		float pitch = 1;
		switch (type) {
		case SliderType.HEXAGON:
			pitch = Controller._.hexTarget.x;
			break;
		case SliderType.CIRCLE:
			pitch = Controller._.circTarget.x;
			break;
		case SliderType.TRIANGLE:
			pitch = Controller._.triTarget.x;
			break;
		}

		vol = Mathf.Clamp01 (vol);
		pitch = Mathf.Clamp (pitch + 2, 0.01f,2f);

		sound.volume = vol;
		sound.pitch = pitch;
	}
}
